@@ -9,6 +9,9 @@
 #include <config.h>
 
 #include <unistd.h>
+#ifdef HAVE_SYS_PRCTL_H
+#include <sys/prctl.h>
+#endif
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/wait.h>
@@ -119,6 +122,9 @@ void generator_manager_base::start()  {
 
 
   // In child
+#ifdef PR_SET_PDEATHSIG
+  prctl(PR_SET_PDEATHSIG, SIGHUP);
+#endif
   if(setup_signal_handlers() == -1)
     exit(EXIT_FAILURE);
   start_commands(); // child start commands
@@ -190,6 +196,9 @@ void generator_manager_base::start_one_command(const std::string& command, int p
   }
 
   // In child
+#ifdef PR_SET_PDEATHSIG
+  prctl(PR_SET_PDEATHSIG, SIGHUP);
+#endif
   int dev_null = open_cloexec("/dev/null", O_RDONLY);
   if(dev_null != -1)
     dup2(dev_null, 0);
