@@ -1,6 +1,6 @@
 #! /bin/sh
 
-cd tests
+mkdir -p tests-data; cd tests-data
 . ../compat.sh
 
 ${DIR}/generate_sequence -v -q -o seq10m -m 19 -s 2718281828 10000000
