@@ -1,6 +1,6 @@
 #! /bin/sh
 
-cd tests
+mkdir -p tests-data; cd tests-data
 . ../compat.sh
 
 ${DIR}/generate_sequence -v -o seq10m -m 10 -m 22 -s 3141592653 10000000
