@@ -8,7 +8,7 @@ TEST(AllocMmap, Simple) {
 
   EXPECT_EQ(size, mem.get_size());
   EXPECT_NE((void*)0, mem.get_ptr());
-  char* ptr = (char*)mem.get_ptr();
+  //  char* ptr = (char*)mem.get_ptr();
   // char c = ptr[size];
   // EXPECT_EQ((char)0, c);
 }
