@@ -66,11 +66,12 @@ TEST(Random, Bits) {
 }
 
 TEST(BinarySearchFirst, Int) {
+  using namespace std::placeholders;
   static int size = 1024;
 
   for(int i = 0; i < size; ++i)
     EXPECT_EQ(i, *jellyfish::binary_search_first_false(jellyfish::pointer_integer<int>(0), jellyfish::pointer_integer<int>(size),
-                                                       std::bind2nd(std::less<int>(), i)));
+                                                       std::bind(std::less<int>(), _1, i)));
 }
 
 TEST(Slices, NonOverlapAll) {
