@@ -12,10 +12,10 @@ struct opened_streams {
   Iterator begin_, end_;
 
   opened_streams(Iterator begin, Iterator end) : begin_(begin), end_(end) { }
-  std::unique_ptr<std::istream> next() {
-    std::unique_ptr<std::istream> res;
+  jellyfish::stream_type next() {
+    jellyfish::stream_type res;
     if(begin_ != end_) {
-      res.reset(*begin_);
+      res.standard.reset(*begin_);
       ++begin_;
     }
     return res;
