@@ -1,7 +1,7 @@
 #! /bin/sh
 
 cd tests
-. ./compat.sh
+. ../compat.sh
 
 if [ -n "$VALGRIND" ]; then
     exec valgrind ${DIR}/test_all
